"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.clipboard = void 0;
const prosemirror_model_1 = require("prosemirror-model");
const prosemirror_state_1 = require("prosemirror-state");
require("../../../types/spec");
const logger_1 = require("../../../logger");
const helpers_1 = require("../../../utils/helpers");
const core_1 = require("../../../core");
const selection_1 = require("../../../utils/selection");
const BaseSchema_1 = require("../../base/BaseSchema");
const code_1 = require("./code");
const utils_1 = require("./utils");
const clipboard = ({ parser, serializer, pasteFileHandler }) => {
    const copyTo = copyToFactory(serializer);
    return new prosemirror_state_1.Plugin({
        props: {
            // @ts-expect-error handleDOMEvents has broken types
            handleDOMEvents: {
                copy(view, e) {
                    if (!e.clipboardData)
                        return false;
                    if (view.state.selection.empty)
                        return false;
                    const slice = getCopyContent(view.state);
                    copyTo(e.clipboardData, slice);
                    e.preventDefault();
                    return true;
                },
                cut(view, e) {
                    if (!e.clipboardData)
                        return false;
                    if (view.state.selection.empty)
                        return false;
                    const slice = getCopyContent(view.state);
                    copyTo(e.clipboardData, slice);
                    view.dispatch(view.state.tr.replaceSelection(prosemirror_model_1.Slice.empty));
                    e.preventDefault();
                    return true;
                },
                paste(view, e) {
                    if (!e.clipboardData)
                        return false;
                    let data;
                    let isPasteHandled = false;
                    if ((0, utils_1.isIosSafariShare)(e.clipboardData) &&
                        (data = e.clipboardData.getData(utils_1.DataTransferType.UriList))) {
                        const schema = view.state.schema;
                        view.dispatch((0, core_1.trackTransactionMetrics)(view.state.tr.replaceSelection(
                        // paste link inline
                        new prosemirror_model_1.Slice(prosemirror_model_1.Fragment.from((0, BaseSchema_1.pType)(schema).create(null, schema.text(data))), 1, 1)), 'paste', { clipboardDataFormat: utils_1.DataTransferType.UriList }));
                        isPasteHandled = true;
                        e.preventDefault();
                        return true;
                    }
                    if ((data = e.clipboardData.getData(utils_1.DataTransferType.Text))) {
                        const codeType = (0, code_1.isInsideCode)(view.state);
                        if (codeType) {
                            const schema = view.state.schema;
                            view.dispatch((0, core_1.trackTransactionMetrics)(view.state.tr.replaceSelectionWith(schema.text(codeType === 'inline' ? data.trim() : data)), 'paste', { clipboardDataFormat: utils_1.DataTransferType.Text, code: codeType }));
                            isPasteHandled = true;
                        }
                        else {
                            const res = (0, helpers_1.tryCatch)(() => parser.parse(data));
                            if (res.success) {
                                const docNode = res.result;
                                const slice = getSliceFromMarkupFragment(docNode.content);
                                view.dispatch((0, core_1.trackTransactionMetrics)(view.state.tr.replaceSelection(slice), 'paste', { clipboardDataFormat: utils_1.DataTransferType.Text }));
                                isPasteHandled = true;
                            }
                            else {
                                logger_1.logger.error(res.error);
                                console.error(res.error);
                            }
                        }
                    }
                    if (e.clipboardData.files.length && pasteFileHandler) {
                        for (const file of Array.from(e.clipboardData.files)) {
                            pasteFileHandler(file);
                        }
                        isPasteHandled = true;
                    }
                    if (isPasteHandled) {
                        e.preventDefault();
                        return true;
                    }
                    return false;
                },
            },
        },
    });
};
exports.clipboard = clipboard;
function getSliceFromMarkupFragment(fragment) {
    var _a;
    let start = 0;
    let end = 0;
    if ((_a = fragment.firstChild) === null || _a === void 0 ? void 0 : _a.isTextblock) {
        start = 1;
        if (fragment.childCount === 1) {
            end = 1;
        }
    }
    return new prosemirror_model_1.Slice(fragment, start, end);
}
function copyToFactory(serializer) {
    return (data, slice) => {
        data.setData(utils_1.DataTransferType.Text, serializer.serialize(slice.content));
    };
}
function getCopyContent(state) {
    var _a;
    const sel = state.selection;
    if ((0, selection_1.isWholeSelection)(sel)) {
        return new prosemirror_model_1.Slice(state.doc.content, 0, 0);
    }
    if (isTextSelectionWithinSameTextBlock(sel)) {
        return new prosemirror_model_1.Slice(createFragmentFromInlineSelection(state), 0, 0);
    }
    if ((0, selection_1.isNodeSelection)(sel)) {
        const { node } = sel;
        if (node.type.spec.complex) {
            if (node.isTextblock) {
                return new prosemirror_model_1.Slice(prosemirror_model_1.Fragment.from(state.schema.node(BaseSchema_1.BaseNode.Paragraph, {}, node.content)), 1, 1);
            }
            else {
                const { complex } = node.type.spec;
                if (complex === 'root') {
                    return new prosemirror_model_1.Slice(prosemirror_model_1.Fragment.from(node), 1, 1);
                }
                else if (complex === 'leaf') {
                    return new prosemirror_model_1.Slice(prosemirror_model_1.Fragment.from(node.content), 1, 1);
                }
                else {
                    // TODO: handle copy of intermediate complex block node
                }
            }
        }
        else {
            return new prosemirror_model_1.Slice(prosemirror_model_1.Fragment.from(node), 1, 1);
        }
    }
    // selection.content() get smart slice – it copy a structure of complex nodes
    // e.g. if select part of cut title it creates slice with yfm-cut –> yfm-cut-title -> selected text
    // it works well with simple nodes, but to handle cases as described above, custom logic nedded
    let slice = sel.content();
    // replace first node with paragraph if needed
    if (((_a = slice.content.firstChild) === null || _a === void 0 ? void 0 : _a.isTextblock) &&
        slice.content.firstChild.type.name !== BaseSchema_1.BaseNode.Paragraph) {
        let shouldTransformFirstChildToPara = true;
        const { firstChild } = slice.content;
        // if all text selected inside first node, don't replace it with paragraph
        if (firstChild.eq(sel.$from.parent)) {
            shouldTransformFirstChildToPara = false;
        }
        if (shouldTransformFirstChildToPara) {
            slice = new prosemirror_model_1.Slice(slice.content.replaceChild(0, state.schema.node(BaseSchema_1.BaseNode.Paragraph, {}, firstChild.content)), slice.openStart, slice.openStart);
        }
    }
    return slice;
}
function isTextSelectionWithinSameTextBlock(sel) {
    return (0, selection_1.isTextSelection)(sel) && sel.$from.sameParent(sel.$to) && sel.$from.parent.isTextblock;
}
function createFragmentFromInlineSelection(state) {
    const sel = state.selection;
    const inlineSlice = state.doc.slice(sel.from, sel.to);
    const isComplexTextblock = Boolean(sel.$from.parent.type.spec.complex);
    const isAllContentSelected = sel.$from.parent.content.eq(inlineSlice.content);
    if (isComplexTextblock || !isAllContentSelected) {
        // copy selected inline content to paragraph if current node is complex
        // or if only part of it's content was selected
        return prosemirror_model_1.Fragment.from(state.schema.node(BaseSchema_1.BaseNode.Paragraph, {}, inlineSlice.content));
    }
    return prosemirror_model_1.Fragment.from(sel.$from.parent.copy(inlineSlice.content));
}
