"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.needParagraphAfter = exports.needParagraphBefore = exports.arrowRight = exports.arrowUp = exports.arrowDown = exports.arrowLeft = exports.createFakeParagraphNear = exports.Direction = void 0;
const prosemirror_utils_1 = require("prosemirror-utils");
const prosemirror_state_1 = require("prosemirror-state");
const nodes_1 = require("../../../utils/nodes");
const GapCursorSelection_1 = require("../../behavior/Cursor/GapCursorSelection");
var Direction;
(function (Direction) {
    Direction["up"] = "up";
    Direction["right"] = "right";
    Direction["left"] = "left";
    Direction["down"] = "down";
})(Direction = exports.Direction || (exports.Direction = {}));
const isUp = (dir) => [Direction.up, Direction.left].includes(dir);
const findEdgeNode = (n) => n.type.spec.complex === 'root' || // code block is exceptional
    (0, nodes_1.isCodeBlock)(n);
const arrow = (dir) => (state, dispatch) => {
    const { selection } = state;
    const pos = state.doc.resolve(selection.head);
    if (selection instanceof prosemirror_state_1.TextSelection || selection instanceof GapCursorSelection_1.GapCursorSelection) {
        const parent = (0, prosemirror_utils_1.findParentNodeClosestToPos)(pos, findEdgeNode);
        if (isUp(dir) && (0, exports.needParagraphBefore)(state, dir)) {
            return (0, exports.createFakeParagraphNear)(Direction.up, parent)(state, dispatch);
        }
        if (!isUp(dir) && (0, exports.needParagraphAfter)(state, dir)) {
            return (0, exports.createFakeParagraphNear)(Direction.down, parent)(state, dispatch);
        }
    }
    if (selection instanceof prosemirror_state_1.NodeSelection) {
        if (selection.node.isInline)
            return false;
        return (0, exports.createFakeParagraphNear)(isUp(dir) ? Direction.up : Direction.down, {
            pos: pos.pos - 1,
            node: selection.$from.parent,
        })(state, dispatch);
    }
    return false;
};
const createFakeParagraphNear = (direction, parent) => (state, dispatch) => {
    const paragraph = state.schema.nodes.paragraph;
    if (!paragraph || !parent) {
        return false;
    }
    const insertPos = direction === 'up' ? parent.pos : parent.pos + parent.node.nodeSize;
    const tr = state.tr;
    const sel = new GapCursorSelection_1.GapCursorSelection(tr.doc.resolve(insertPos));
    tr.setSelection(sel);
    if (dispatch) {
        dispatch(tr);
    }
    return true;
};
exports.createFakeParagraphNear = createFakeParagraphNear;
exports.arrowLeft = arrow(Direction.left);
exports.arrowDown = arrow(Direction.down);
exports.arrowUp = arrow(Direction.up);
exports.arrowRight = arrow(Direction.right);
const needParagraphBefore = (state, dir) => {
    if (!isUp(dir))
        return false;
    const { selection } = state;
    const { $from } = selection;
    const pos = state.doc.resolve(selection.head);
    const disabledParent = (0, prosemirror_utils_1.findParentNodeClosestToPos)(pos, (n) => n.type.spec.disableGapCursor);
    if (disabledParent) {
        return false;
    }
    const parent = (0, prosemirror_utils_1.findParentNodeClosestToPos)(pos, findEdgeNode);
    if (parent) {
        if (state.doc.resolve(Math.max(parent.pos - 1, 0)).parent.type.name === 'paragraph') {
            return false;
        }
        const firstTextBlock = (0, prosemirror_utils_1.findChildren)(parent.node, (n) => n.isTextblock || n.isText)[0];
        return (parent.start === $from.pos - ($from.depth - parent.depth) ||
            (dir === 'up' &&
                $from.pos < parent.start + firstTextBlock.pos + firstTextBlock.node.nodeSize));
    }
    return false;
};
exports.needParagraphBefore = needParagraphBefore;
const needParagraphAfter = (state, dir) => {
    if (isUp(dir))
        return false;
    const { selection } = state;
    const { $from } = selection;
    const pos = state.doc.resolve(selection.head);
    const disabledParent = (0, prosemirror_utils_1.findParentNodeClosestToPos)(pos, (n) => n.type.spec.disableGapCursor);
    if (disabledParent) {
        return false;
    }
    const parent = (0, prosemirror_utils_1.findParentNodeClosestToPos)(pos, findEdgeNode);
    if (parent) {
        if (state.doc.resolve(Math.min(parent.pos + parent.node.nodeSize + 1, state.doc.nodeSize - 2)).parent.type.name === 'paragraph') {
            return false;
        }
        const children = (0, prosemirror_utils_1.findChildren)(parent.node, (n) => n.isTextblock || n.isText);
        const lastTextBlock = children[children.length - 1];
        return (parent.pos + parent.node.nodeSize === $from.pos + ($from.depth - parent.depth) + 1 ||
            (dir === 'down' && $from.pos > parent.start + lastTextBlock.pos));
    }
    return false;
};
exports.needParagraphAfter = needParagraphAfter;
