"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.inlineNodeInputRule = exports.nodeInputRule = exports.markInputRule = void 0;
const prosemirror_inputrules_1 = require("prosemirror-inputrules");
const prosemirror_model_1 = require("prosemirror-model");
const prosemirror_state_1 = require("prosemirror-state");
const marks_1 = require("../utils/marks");
// TODO: remove explicit import from code extension
const extensions_1 = require("../extensions");
function getMarksBetween(start, end, state) {
    let marks = [];
    state.doc.nodesBetween(start, end, (node, pos) => {
        marks = [
            ...marks,
            ...node.marks.map((mark) => ({
                start: pos,
                end: pos + node.nodeSize,
                mark,
            })),
        ];
    });
    return marks;
}
function escapeRegex(string) {
    // eslint-disable-next-line no-useless-escape
    return string.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
}
// TODO: link has a custom input rule handler, think about generalizing
function markInputRule(rule, markType, getAttrs) {
    let regexp;
    if (rule instanceof RegExp) {
        regexp = rule;
    }
    else {
        const open = escapeRegex(rule.open);
        const close = escapeRegex(rule.close);
        const ignoreBetween = escapeRegex(rule.ignoreBetween || '');
        regexp = new RegExp(`(?:${open})` +
            (ignoreBetween ? `([^\\s${ignoreBetween}]+)` : `([\\S]+)`) +
            `(?:${close})\\s$`);
    }
    return new prosemirror_inputrules_1.InputRule(regexp, (state, match, start, end) => {
        // handle the rule only if is start of line or there is a space before "open" symbols
        if (match.index > 0) {
            const re = match;
            if (re.input[re.index - 1] !== ' ')
                return null;
        }
        // TODO: remove explicit import from code extension
        const codeMarkType = (0, extensions_1.codeType)(state.schema);
        if ((0, marks_1.isMarkActive)(state, codeMarkType)) {
            return null;
        }
        const attrs = getAttrs instanceof Function ? getAttrs(match) : getAttrs;
        const { tr } = state;
        const m = match.length - 1;
        let markEnd = end;
        let markStart = start;
        if (match[m]) {
            const matchStart = start + match[0].indexOf(match[m - 1]);
            const matchEnd = matchStart + match[m - 1].length - 1;
            const textStart = matchStart + match[m - 1].lastIndexOf(match[m]);
            const textEnd = textStart + match[m].length;
            const marksBetween = getMarksBetween(start, end, state);
            if (marksBetween.some((item) => item.mark.type === codeMarkType)) {
                return null;
            }
            const excludedMarks = marksBetween
                .filter((item) => item.mark.type.excludes(markType))
                .filter((item) => item.end > matchStart);
            if (excludedMarks.length) {
                return null;
            }
            if (textEnd < matchEnd) {
                tr.delete(textEnd, matchEnd);
            }
            if (textStart > matchStart) {
                tr.delete(matchStart, textStart);
            }
            markStart = matchStart;
            markEnd = markStart + match[m].length;
        }
        tr.addMark(markStart, markEnd, markType.create(attrs));
        tr.removeStoredMark(markType);
        return tr;
    });
}
exports.markInputRule = markInputRule;
function nodeInputRule(regexp, fragment, selectionOffset = 0) {
    return new prosemirror_inputrules_1.InputRule(regexp, (state, match, start, end) => {
        const [matchStr] = match;
        const { tr } = state;
        if (matchStr) {
            tr.replaceWith(start - 1, end, fragment || prosemirror_model_1.Fragment.empty).setSelection(new prosemirror_state_1.TextSelection(tr.doc.resolve(start + selectionOffset)));
        }
        return tr;
    });
}
exports.nodeInputRule = nodeInputRule;
function inlineNodeInputRule(regexp, fragment) {
    return new prosemirror_inputrules_1.InputRule(regexp, (state, match, start, end) => {
        var _a;
        const [matchStr] = match;
        const { tr } = state;
        if (matchStr) {
            tr.replaceWith(start, end, (_a = fragment(matchStr)) !== null && _a !== void 0 ? _a : prosemirror_model_1.Fragment.empty).setSelection(new prosemirror_state_1.TextSelection(tr.doc.resolve(end)));
        }
        return tr;
    });
}
exports.inlineNodeInputRule = inlineNodeInputRule;
